-- global variables
local current_dungeon_string = ""
local current_steps = 0
local timer
local update = false
local addon_loaded = false
local lock = false

-- called upon loading MainFrame
function Superluminal_OnLoad()
	print("Superluminal loaded. This addon is in very early stages of development. Please report any bugs you find to julianneeleman@hotmail.com.")
	MainFrame:RegisterEvent("ADDON_LOADED")
	MainFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
	MainFrame:RegisterEvent("CHALLENGE_MODE_COMPLETED")
	MainFrame:RegisterEvent("SCENARIO_CRITERIA_UPDATE")
	MainFrame:RegisterEvent("WORLD_STATE_TIMER_START")
	MainFrame:RegisterEvent("WORLD_STATE_TIMER_STOP")
	if current_run == nil then
		current_run = {["total"] = 0}
	end
end

-- sets split timers
function Superluminal_UpdateSplitTimes()
	local split_times_string = ""
	for i = 1, current_steps do
		split_times_string = split_times_string .. Superluminal_Time_To_String(best_runs[current_dungeon_string][i]) .. "\n"
	end
	SplitTimesFrame_SplitTimesString:SetText(split_times_string)
end

-- updates the MainFrame's content (objectives/progress)
function Superluminal_UpdateObjectives()		
	local new_objectives_string = ""
	local new_objective_times_string = ""
	for i = 1, current_steps do
		-- for every objective, add the string to the objectives string
		local name, _, status, progress, required, _, _, _, _, time_since_completion = C_Scenario.GetCriteriaInfo(i)
		new_objectives_string = new_objectives_string .. progress .. "/" .. required .. " " .. name .. "\n"	
	end
	ObjectivesFrame_ObjectivesString:SetText(new_objectives_string)
end

-- updates timers
function Superluminal_OnUpdate(self, elapsed)
	if update and C_Scenario.IsChallengeMode() then
		_, timer = GetWorldElapsedTime(1)
		if timer > 0 then
			current_run["total"] = timer
			TimerFrame_TimerString:SetText(Superluminal_Time_To_String(timer))
			
			local new_objective_times_string = ""		
			for i = 1, current_steps do
				local name, _, status, progress, required, _, _, _, _, time_since_completion = C_Scenario.GetCriteriaInfo(i)
				if current_run[i] == nil then
					if status then	
						current_run[i] = timer + time_since_completion
						new_objective_times_string = new_objective_times_string .. Superluminal_Time_To_String(current_run[i])
						if best_runs[current_dungeon_string][i] ~= "NV" then
							new_objective_times_string = new_objective_times_string .. " (" .. Superluminal_Time_To_CString(best_runs[current_dungeon_string][i] - current_run[i]) .. ")"
						end
					elseif best_runs[current_dungeon_string][i] ~= "NV" then
						new_objective_times_string = new_objective_times_string .. Superluminal_Time_To_CString(best_runs[current_dungeon_string][i] - timer)
					end
				else
					new_objective_times_string = new_objective_times_string .. Superluminal_Time_To_String(current_run[i])
					if best_runs[current_dungeon_string][i] ~= "NV" then
						new_objective_times_string = new_objective_times_string .. " (" .. Superluminal_Time_To_CString(best_runs[current_dungeon_string][i] - current_run[i]) .. ")"
					end
				end
				new_objective_times_string = new_objective_times_string .. "\n"	
			end
			ObjectiveTimesFrame_ObjectiveTimesString:SetText(new_objective_times_string)
		end
	end
end

-- stores a new best run in memory if it beats the one currently in memory
function Superluminal_OnChallengeModeCompleted()
	update = false
	if best_runs[current_dungeon_string]["total"] == "NV" or current_run["total"] < best_runs[current_dungeon_string]["total"] then
		best_runs[current_dungeon_string]["total"] = current_run["total"]
		for i = 1, current_steps do
			if current_run[i] == nil then
				best_runs[current_dungeon_string][i] = current_run["total"]
			else
				best_runs[current_dungeon_string][i] = current_run[i]
			end
		end
	end
end

-- initiates best run values when a challenge mode is entered, then loads split timers and objectives
function Superluminal_PlayerEnteringChallengeMode()
	current_dungeon_string, _, current_steps = C_Scenario.GetStepInfo()
	if best_runs == nil then
		best_runs = {}
	end
	if best_runs[current_dungeon_string] == nil then
		best_runs[current_dungeon_string] = {}
		best_runs[current_dungeon_string]["total"] = "NV"
		for i = 1, current_steps do
			best_runs[current_dungeon_string][i] = "NV"
		end
	end
	update = true
	Superluminal_UpdateSplitTimes()
	Superluminal_UpdateObjectives()
end

function Superluminal_OnAddonLoaded(name)
	if name == "Superluminal" then
		addon_loaded = true
	end
end

-- event handler
function Superluminal_OnEvent(self, event, ...)
	if event == "ADDON_LOADED" then
		Superluminal_OnAddonLoaded(...)
	end
	if C_Scenario.IsChallengeMode() and addon_loaded then	
		MainFrame:Show()
		WatchFrame:Hide()		
		if event == "PLAYER_ENTERING_WORLD" then
			update = false
			Superluminal_PlayerEnteringChallengeMode()
		elseif event == "SCENARIO_CRITERIA_UPDATE" then
			Superluminal_UpdateObjectives() -- updates MainFrame whenever an objective is updated
		elseif event == "CHALLENGE_MODE_COMPLETED" then
			Superluminal_OnChallengeModeCompleted()
		elseif event == "WORLD_STATE_TIMER_START" then
			update = true
			current_run = {["total"] = 0}
		elseif event == "WORLD_STATE_TIMER_STOP" then
			update = false
			Superluminal_UpdateSplitTimes()
		end
	else
		MainFrame:Hide() -- hide MainFrame while not in a challenge mode
		WatchFrame:Show()
	end
end

-- drag & drop functions
function Superluminal_OnMouseDown()
	if not lock then
		MainFrame:StartMoving()
	end
end

function Superluminal_OnMouseUp()
	MainFrame:StopMovingOrSizing()
end

-- general conversion function for time to string
function Superluminal_Time_To_String(time_value)
	if time_value == "NV" then
		return time_value
	end
	local minutes = floor(abs(time_value) / 60)
	local seconds = abs(time_value) % 60
	local negative_sign = ""
	local optional_zero = ""
	if time_value < 0 then
		negative_sign = "-"
	end
	if seconds < 10 then
		optional_zero = "0" -- prevents single-digit second-timers from looking odd (12:4 -> 12:04)
	end
	return minutes .. ":" .. optional_zero .. seconds
end

-- conversion function for comparison times, signed (+/-)
function Superluminal_Time_To_CString(time_value)
	if time_value == "NV" then
		return time_value
	end
	local minutes = floor(abs(time_value) / 60)
	local seconds = abs(time_value) % 60
	local sign = "+"
	local optional_zero = ""
	if time_value < 0 then
		sign = "-"
	end
	if seconds < 10 then
		optional_zero = "0" -- prevents single-digit second-timers from looking odd (12:4 -> 12:04)
	end
	return sign .. minutes .. ":" .. optional_zero .. seconds
end

SLASH_SUPERLUMINAL1, SLASH_SUPERLUMINAL2 = "/sl", "/superluminal";
function SlashCmdList.SUPERLUMINAL(msg, editbox)
	if msg == "lock" then
		lock = not lock
	end
end